@@ -275,11 +275,6 @@ public:
         ASR::dimension_t* m_dims = nullptr;
         size_t n_dims = ASRUtils::extract_dimensions_from_ttype(v.m_type, m_dims);
         ASR::ttype_t* v_m_type = v.m_type;
-        if (ASR::is_a<ASR::Const_t>(*v_m_type)) {
-            if( is_array ) {
-                v_m_type = ASR::down_cast<ASR::Const_t>(v_m_type)->m_type;
-            }
-        }
         v_m_type = ASRUtils::type_get_past_array(ASRUtils::type_get_past_allocatable(v_m_type));
         if (ASRUtils::is_pointer(v_m_type)) {
             ASR::ttype_t *t2 = ASR::down_cast<ASR::Pointer_t>(v_m_type)->m_type;
@@ -400,7 +395,13 @@ public:
         } else {
             std::string dims;
             use_ref = use_ref && !is_array;
-            if (ASRUtils::is_integer(*v_m_type)) {
+            if (v.m_storage == ASR::storage_typeType::Parameter) {
+                convert_variable_decl_util(v, is_array, declare_as_constant, use_ref, dummy,
+                    force_declare, force_declare_name, n_dims, m_dims, v_m_type, dims, sub);
+                if (v.m_intent != ASR::intentType::ReturnVar) {
+                    sub = "const " + sub;
+                }
+            } else if (ASRUtils::is_integer(*v_m_type)) {
                 headers.insert("inttypes.h");
                 convert_variable_decl_util(v, is_array, declare_as_constant, use_ref, dummy,
                     force_declare, force_declare_name, n_dims, m_dims, v_m_type, dims, sub);
@@ -415,11 +416,6 @@ public:
                 headers.insert("complex.h");
                 convert_variable_decl_util(v, is_array, declare_as_constant, use_ref, dummy,
                     force_declare, force_declare_name, n_dims, m_dims, v_m_type, dims, sub);
-            } else if (ASR::is_a<ASR::SymbolicExpression_t>(*v_m_type)) {
-                headers.insert("symengine/cwrapper.h");
-                std::string type_name = "basic";
-                std::string v_m_name = v.m_name;
-                sub = format_type_c("", type_name, v_m_name, use_ref, dummy);
             } else if (ASRUtils::is_logical(*v_m_type)) {
                 convert_variable_decl_util(v, is_array, declare_as_constant, use_ref, dummy,
                     force_declare, force_declare_name, n_dims, m_dims, v_m_type, dims, sub);
@@ -529,7 +525,11 @@ public:
             } else if (ASR::is_a<ASR::List_t>(*v_m_type)) {
                 ASR::List_t* t = ASR::down_cast<ASR::List_t>(v_m_type);
                 std::string list_type_c = c_ds_api->get_list_type(t);
-                sub = format_type_c("", list_type_c, v.m_name,
+                std::string name = v.m_name;
+                if (v.m_intent == ASRUtils::intent_out) {
+                    name = "*" + name;
+                }
+                sub = format_type_c("", list_type_c, name,
                                     false, false);
             } else if (ASR::is_a<ASR::Tuple_t>(*v_m_type)) {
                 ASR::Tuple_t* t = ASR::down_cast<ASR::Tuple_t>(v_m_type);
@@ -547,11 +547,6 @@ public:
                 ASR::Enum_t* enum_ = ASR::down_cast<ASR::Enum_t>(v_m_type);
                 ASR::EnumType_t* enum_type = ASR::down_cast<ASR::EnumType_t>(enum_->m_enum_type);
                 sub = format_type_c("", "enum " + std::string(enum_type->m_name), v.m_name, false, false);
-            } else if (ASR::is_a<ASR::Const_t>(*v_m_type)) {
-                std::string const_underlying_type = CUtils::get_c_type_from_ttype_t(
-                    ASRUtils::type_get_past_const(v_m_type));
-                sub = format_type_c("", "const " + const_underlying_type,
-                                    v.m_name, false, false);
             } else if (ASR::is_a<ASR::TypeParameter_t>(*v_m_type)) {
                 // Ignore type variables
                 return "";
@@ -566,7 +561,7 @@ public:
             }
             if (dims.size() == 0 && v.m_symbolic_value && !do_not_initialize) {
                 ASR::expr_t* init_expr = v.m_symbolic_value;
-                if( !ASR::is_a<ASR::Const_t>(*v.m_type) ) {
+                if( v.m_storage != ASR::storage_typeType::Parameter ) {
                     for( size_t i = 0; i < v.n_dependencies; i++ ) {
                         std::string variable_name = v.m_dependencies[i];
                         ASR::symbol_t* dep_sym = current_scope->resolve_symbol(variable_name);
@@ -1123,8 +1118,7 @@ R"(    // Initialise Numpy
             for (int i = n_dims - 1; i >= 0; i--) {
                 std::string start = "0", length = "0";
                 if( lower_bounds ) {
-                    visit_expr(*lower_bounds->m_args[i]);
-                    start = src;
+                    start = ASRUtils::fetch_ArrayConstant_value(lower_bounds, i);
                 }
                 if( m_dims[i].m_length ) {
                     this->visit_expr(*m_dims[i].m_length);
@@ -1301,9 +1295,8 @@ R"(    // Initialise Numpy
         // TODO: Support and test for multi-dimensional array constants
         headers.insert("stdarg.h");
         std::string array_const = "";
-        for( size_t i = 0; i < x.n_args; i++ ) {
-            visit_expr(*x.m_args[i]);
-            array_const += src + ", ";
+        for( size_t i = 0; i < (size_t) ASRUtils::get_fixed_size_of_array(x.m_type); i++ ) {
+            array_const += ASRUtils::fetch_ArrayConstant_value(x, i) + ", ";
         }
         array_const.pop_back();
         array_const.pop_back();
@@ -1314,7 +1307,7 @@ R"(    // Initialise Numpy
         std::string return_type = c_ds_api->get_array_type(array_type_name, array_encoded_type_name,array_types_decls, false);
 
         src = c_utils_functions->get_array_constant(return_type, array_type_name, array_encoded_type_name) +
-                "(" + std::to_string(x.n_args) + ", " + array_const + ")";
+                "(" + std::to_string(ASRUtils::get_fixed_size_of_array(x.m_type)) + ", " + array_const + ")";
     }
 
     void visit_ArrayItem(const ASR::ArrayItem_t &x) {
@@ -1368,7 +1361,7 @@ R"(    // Initialise Numpy
         }
 
         ASR::ttype_t* x_mv_type_ = ASRUtils::type_get_past_allocatable(
-                ASRUtils::type_get_past_pointer(ASRUtils::type_get_past_const(x_mv_type)));
+                ASRUtils::type_get_past_pointer(x_mv_type));
         LCOMPILERS_ASSERT(ASR::is_a<ASR::Array_t>(*x_mv_type_));
         ASR::Array_t* array_t = ASR::down_cast<ASR::Array_t>(x_mv_type_);
         std::vector<std::string> diminfo;
