@@ -126,13 +126,13 @@ class PassArrayByDataProcedureVisitor : public PassUtils::PassVisitor<PassArrayB
                 } else {
                     break ;
                 }
-                if( std::find(indices.begin(), indices.end(), i) !=
-                    indices.end() ) {
+                if( std::find(indices.begin(), indices.end(), i) != indices.end() ) {
                     if( arg_func ) {
-                        suffix += "_" + std::string(arg_func->m_name);
+                        suffix += "_" + ASRUtils::type_to_str(arg_func->m_function_signature);
                     } else {
-                        suffix += "_" + std::string(arg->m_name);
+                        suffix += "_" + ASRUtils::type_to_str(arg->m_type);
                     }
+                    suffix += "_" + std::to_string(i);
                 }
                 ASR::expr_t* new_arg;
                 if (arg_func) {
@@ -151,6 +151,13 @@ class PassArrayByDataProcedureVisitor : public PassUtils::PassVisitor<PassArrayB
                 }
             }
             ASR::symbol_t* new_symbol = nullptr;
+            suffix = to_lower(suffix);
+            for( size_t suffixi = 0; suffixi < suffix.size(); suffixi++ ) {
+                if( !((suffix[suffixi] >= 'a' && suffix[suffixi] <= 'z') ||
+                    (suffix[suffixi] >= '0' && suffix[suffixi] <= '9')) ) {
+                    suffix[suffixi] = '_';
+                }
+            }
             std::string new_name = std::string(x->m_name) + suffix;
             if( ASR::is_a<ASR::Function_t>( *((ASR::symbol_t*) x) ) ) {
                 ASR::FunctionType_t* x_func_type = ASRUtils::get_FunctionType(x);
@@ -373,6 +380,12 @@ class EditProcedureVisitor: public ASR::CallReplacerOnExpressionsVisitor<EditPro
         ASR::CallReplacerOnExpressionsVisitor<EditProcedureVisitor>::visit_BlockCall(x);
     }
 
+    void visit_AssociateBlockCall(const ASR::AssociateBlockCall_t& x) {
+        ASR::AssociateBlockCall_t& xx = const_cast<ASR::AssociateBlockCall_t&>(x);
+        edit_symbol_reference(m)
+        ASR::CallReplacerOnExpressionsVisitor<EditProcedureVisitor>::visit_AssociateBlockCall(x);
+    }
+
     void visit_SubroutineCall(const ASR::SubroutineCall_t& x) {
         ASR::SubroutineCall_t& xx = const_cast<ASR::SubroutineCall_t&>(x);
         edit_symbol_reference(name)
@@ -444,6 +457,29 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
             }
         }
 
+        static inline void get_dimensions(ASR::expr_t* array, Vec<ASR::expr_t*>& dims,
+                                        Allocator& al) {
+            ASR::ttype_t* array_type = ASRUtils::expr_type(array);
+            ASR::dimension_t* compile_time_dims = nullptr;
+            int n_dims = ASRUtils::extract_dimensions_from_ttype(array_type, compile_time_dims);
+            for( int i = 0; i < n_dims; i++ ) {
+                ASR::expr_t* start = compile_time_dims[i].m_start;
+                if( start == nullptr ) {
+                    start = PassUtils::get_bound(array, i + 1, "lbound", al);
+                }
+                ASR::expr_t* length = compile_time_dims[i].m_length;
+                if( length == nullptr ) {
+                    if (ASRUtils::is_allocatable(array_type)) {
+                        length = ASRUtils::get_size(array, i + 1, al);
+                    } else {
+                        length = PassUtils::get_bound(array, i + 1, "ubound", al);
+                    }
+                }
+                dims.push_back(al, start);
+                dims.push_back(al, length);
+            }
+        }
+
         Vec<ASR::call_arg_t> construct_new_args(size_t n_args, ASR::call_arg_t* orig_args, std::vector<size_t>& indices) {
             Vec<ASR::call_arg_t> new_args;
             new_args.reserve(al, n_args);
@@ -477,7 +513,7 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
 
                 Vec<ASR::expr_t*> dim_vars;
                 dim_vars.reserve(al, 2);
-                ASRUtils::get_dimensions(orig_arg_i, dim_vars, al);
+                get_dimensions(orig_arg_i, dim_vars, al);
                 for( size_t j = 0; j < dim_vars.size(); j++ ) {
                     ASR::call_arg_t dim_var;
                     dim_var.loc = dim_vars[j]->base.loc;
@@ -537,12 +573,10 @@ class EditProcedureCallsVisitor : public ASR::ASRPassBaseWalkVisitor<EditProcedu
                 }
                 if( !(min_args <= new_args.size() &&
                     new_args.size() <= max_args) ) {
-                    throw LCompilersException("Number of arguments in the new "
-                                            "function call doesn't satisfy "
-                                            "min_args <= new_args.size() <= max_args, " +
-                                            std::to_string(min_args) + " <= " +
-                                            std::to_string(new_args.size()) + " <= " +
-                                            std::to_string(max_args));
+                    throw LCompilersException("Number of arguments in the new function call " +
+                        std::string(ASRUtils::symbol_name(subrout_sym)) + " doesn't satisfy "
+                        "min_args <= new_args.size() <= max_args, " + std::to_string(min_args) + " <= " +
+                        std::to_string(new_args.size()) + " <= " + std::to_string(max_args));
                 }
             }
             ASR::symbol_t* new_func_sym_ = new_func_sym;
@@ -614,11 +648,30 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
             std::vector<std::string> to_be_erased;
 
             for( auto& item: current_scope->get_scope() ) {
-                if( v.proc2newproc.find(item.second) != v.proc2newproc.end() &&
-                    not_to_be_erased.find(item.second) == not_to_be_erased.end() ) {
+                if (ASR::is_a<ASR::Function_t>(*item.second)) {
+                    ASR::FunctionType_t* func_type = ASRUtils::get_FunctionType(item.second);
+                    if (func_type->n_arg_types >= 1 && ASR::is_a<ASR::Class_t>(*func_type->m_arg_types[0])) {
+                        continue;
+                    }
+                }
+                ASR::symbol_t* sym = item.second;
+                if (ASR::is_a<ASR::ExternalSymbol_t>(*item.second)) {
+                    sym = ASR::down_cast<ASR::ExternalSymbol_t>(item.second)->m_external;
+                }
+                if( v.proc2newproc.find(sym) != v.proc2newproc.end() &&
+                    not_to_be_erased.find(sym) == not_to_be_erased.end() ) {
                     LCOMPILERS_ASSERT(item.first == ASRUtils::symbol_name(item.second))
                     to_be_erased.push_back(item.first);
                 }
+                if ( ASR::is_a<ASR::Module_t>(*item.second) ||
+                    ASR::is_a<ASR::Program_t>(*item.second) ||
+                    ASR::is_a<ASR::Function_t>(*item.second) ||
+                    ASR::is_a<ASR::StructType_t>(*item.second) ||
+                    ASR::is_a<ASR::Block_t>(*item.second)) {
+                    SymbolTable* current_scope_copy = current_scope;
+                    visit_symbol(*item.second);
+                    current_scope = current_scope_copy;
+                }
             }
 
             for (auto &item: to_be_erased) {
@@ -638,6 +691,22 @@ class RemoveArrayByDescriptorProceduresVisitor : public PassUtils::PassVisitor<R
             visit_Unit(x);
         }
 
+        void visit_Module(const ASR::Module_t& x) {
+            if (x.m_intrinsic) {
+                return ;
+            }
+
+            visit_Unit(x);
+        }
+
+        void visit_StructType(const ASR::StructType_t& x) {
+            visit_Unit(x);
+        }
+
+        void visit_Block(const ASR::Block_t& x) {
+            visit_Unit(x);
+        }
+
 };
 
 void pass_array_by_data(Allocator &al, ASR::TranslationUnit_t &unit,
