@@ -92,7 +92,7 @@ public:
         loop_body.push_back(al, loop_stmt);
 
         loop_stmt = ASRUtils::STMT(ASR::make_WhileLoop_t(
-            al, loc, nullptr, loop_test, loop_body.p, loop_body.n));
+            al, loc, nullptr, loop_test, loop_body.p, loop_body.n, nullptr, 0));
         body.push_back(al, loop_stmt);
     }
 
@@ -409,7 +409,7 @@ public:
         }
         ASR::symbol_t *fn_sym = list_section_func_map[list_type_name];
         *current_expr = ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-            fn_sym, nullptr, args.p, args.n, x->m_type, nullptr, nullptr));
+            fn_sym, nullptr, args.p, args.n, x->m_type, nullptr, nullptr, false));
     }
 
     void create_concat_function(Location& loc,
@@ -533,7 +533,7 @@ public:
         }
         ASR::symbol_t *fn_sym = list_concat_func_map[list_type_name];
         *current_expr = ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-            fn_sym, nullptr, args.p, 2, x->m_type, nullptr, nullptr));
+            fn_sym, nullptr, args.p, 2, x->m_type, nullptr, nullptr, false));
     }
 
 };
