@@ -16,6 +16,7 @@
 
 #include <libasr/pass/replace_do_loops.h>
 #include <libasr/pass/replace_for_all.h>
+#include <libasr/pass/while_else.h>
 #include <libasr/pass/replace_init_expr.h>
 #include <libasr/pass/replace_implied_do_loops.h>
 #include <libasr/pass/replace_array_op.h>
@@ -30,6 +31,7 @@
 #include <libasr/pass/replace_div_to_mul.h>
 #include <libasr/pass/replace_symbolic.h>
 #include <libasr/pass/replace_intrinsic_function.h>
+#include <libasr/pass/replace_intrinsic_subroutine.h>
 #include <libasr/pass/replace_fma.h>
 #include <libasr/pass/loop_unroll.h>
 #include <libasr/pass/replace_sign_from_value.h>
@@ -51,6 +53,7 @@
 #include <libasr/pass/insert_deallocate.h>
 #include <libasr/pass/replace_print_struct_type.h>
 #include <libasr/pass/promote_allocatable_to_nonallocatable.h>
+#include <libasr/pass/replace_function_call_in_declaration.h>
 #include <libasr/codegen/asr_to_fortran.h>
 #include <libasr/asr_verify.h>
 #include <libasr/pickle.h>
@@ -74,17 +77,19 @@ namespace LCompilers {
         std::vector<std::string> _skip_passes, _c_skip_passes;
         std::map<std::string, pass_function> _passes_db = {
             {"do_loops", &pass_replace_do_loops},
+            {"while_else", &pass_while_else},
             {"global_stmts", &pass_wrap_global_stmts},
             {"implied_do_loops", &pass_replace_implied_do_loops},
             {"array_op", &pass_replace_array_op},
             {"symbolic", &pass_replace_symbolic},
+            {"flip_sign", &pass_replace_flip_sign},
             {"intrinsic_function", &pass_replace_intrinsic_function},
+            {"intrinsic_subroutine", &pass_replace_intrinsic_subroutine},
             {"arr_slice", &pass_replace_arr_slice},
             {"print_arr", &pass_replace_print_arr},
             {"print_list_tuple", &pass_replace_print_list_tuple},
             {"class_constructor", &pass_replace_class_constructor},
             {"unused_functions", &pass_unused_functions},
-            {"flip_sign", &pass_replace_flip_sign},
             {"div_to_mul", &pass_replace_div_to_mul},
             {"fma", &pass_replace_fma},
             {"sign_from_value", &pass_replace_sign_from_value},
@@ -102,13 +107,13 @@ namespace LCompilers {
             {"init_expr", &pass_replace_init_expr},
             {"nested_vars", &pass_nested_vars},
             {"where", &pass_replace_where},
+            {"function_call_in_declaration", &pass_replace_function_call_in_declaration},
             {"print_struct_type", &pass_replace_print_struct_type},
             {"unique_symbols", &pass_unique_symbols},
             {"insert_deallocate", &pass_insert_deallocate},
             {"promote_allocatable_to_nonallocatable", &pass_promote_allocatable_to_nonallocatable}
         };
 
-        bool is_fast;
         bool apply_default_passes;
         bool c_skip_pass; // This will contain the passes that are to be skipped in C
 
@@ -198,20 +203,23 @@ namespace LCompilers {
             }
         }
 
-        PassManager(): is_fast{false}, apply_default_passes{false},
+        PassManager(): apply_default_passes{false},
             c_skip_pass{false} {
             _passes = {
                 "nested_vars",
                 "global_stmts",
+                "transform_optional_argument_functions",
                 "init_expr",
                 "implied_do_loops",
                 "class_constructor",
                 "pass_list_expr",
                 "where",
+                "function_call_in_declaration",
                 "subroutine_from_function",
                 "array_op",
                 "symbolic",
                 "intrinsic_function",
+                "intrinsic_subroutine",
                 "subroutine_from_function",
                 "array_op",
                 "pass_array_by_data",
@@ -222,10 +230,10 @@ namespace LCompilers {
                 "array_dim_intrinsics_update",
                 "do_loops",
                 "forall",
+                "while_else",
                 "select_case",
                 "inline_function_calls",
                 "unused_functions",
-                "transform_optional_argument_functions",
                 "unique_symbols",
                 "insert_deallocate",
             };
@@ -233,15 +241,19 @@ namespace LCompilers {
             _with_optimization_passes = {
                 "nested_vars",
                 "global_stmts",
+                "transform_optional_argument_functions",
                 "init_expr",
                 "implied_do_loops",
                 "class_constructor",
                 "pass_list_expr",
                 "where",
+                "function_call_in_declaration",
                 "subroutine_from_function",
                 "array_op",
                 "symbolic",
+                "flip_sign",
                 "intrinsic_function",
+                "intrinsic_subroutine",
                 "subroutine_from_function",
                 "array_op",
                 "pass_array_by_data",
@@ -253,14 +265,13 @@ namespace LCompilers {
                 "array_dim_intrinsics_update",
                 "do_loops",
                 "forall",
+                "while_else",
                 "dead_code_removal",
                 "select_case",
                 "unused_functions",
-                "flip_sign",
                 "sign_from_value",
                 "div_to_mul",
                 "fma",
-                "transform_optional_argument_functions",
                 "inline_function_calls",
                 "unique_symbols",
                 "insert_deallocate",
@@ -290,12 +301,10 @@ namespace LCompilers {
                           PassOptions& pass_options,
                           diag::Diagnostics &diagnostics) {
             if( !_user_defined_passes.empty() ) {
-                pass_options.fast = true;
                 apply_passes(al, asr, _user_defined_passes, pass_options,
                     diagnostics);
             } else if( apply_default_passes ) {
-                pass_options.fast = is_fast;
-                if( is_fast ) {
+                if( pass_options.fast ) {
                     apply_passes(al, asr, _with_optimization_passes, pass_options,
                         diagnostics);
                 } else {
@@ -374,14 +383,6 @@ namespace LCompilers {
             }
         }
 
-        void use_optimization_passes() {
-            is_fast = true;
-        }
-
-        void do_not_use_optimization_passes() {
-            is_fast = false;
-        }
-
         void use_default_passes(bool _c_skip_pass=false) {
             apply_default_passes = true;
             c_skip_pass = _c_skip_pass;
