@@ -83,6 +83,8 @@ struct CompilerOptions {
     bool openmp = false;
     bool generate_object_code = false;
     bool no_warnings = false;
+    bool disable_style = false;
+    bool logical_casting = false;
     bool no_error_banner = false;
     bool enable_bounds_checking = false;
     std::string error_format = "human";
@@ -104,6 +106,8 @@ struct CompilerOptions {
     bool legacy_array_sections = false;
     bool ignore_pragma = false;
     bool stack_arrays = false;
+    bool wasm_html = false;
+    std::string emcc_embed;
     std::vector<std::string> import_paths;
     Platform platform;
 
